@@ -9,105 +9,13 @@ from django.http import HttpResponseRedirect
 from django.template import Template
 from django.template.response import TemplateResponse
 from django.urls import path
+from django.utils.text import capfirst
 import time, csv, io
 
-from ..forms import MoveBlocksAdminForm, EmailAdminForm, UserImportForm
+from ..forms import MoveBlocksAdminForm, EmailAdminForm, FormPluginsAdminForm, \
+    UserImportForm, ExportAdminForm
 from ..models import Form, FormBlock, SubmissionRecord
-from ..utils import send_email
-
-
-@admin.action(description='Move to different page')
-def move_blocks_action(modeladmin, request, queryset):
-    if '_move' in request.POST:
-        page, n = request.POST['page'], 0
-        blocks, cur_page = queryset[0].form.blocks, queryset[0].page
-        last_rank = blocks.filter(page=cur_page).aggregate(r=Max('_rank'))['r']
-        for block in queryset.order_by('_rank'):
-            block = FormBlock.objects.get(pk=block.pk) # clear side effects;
-            block._rank = last_rank
-            block.save() # it's now at the end of its page;
-            block.page, block._rank = page, None # and then move to the new one,
-            block.save() # so that the others on the old page are left sorted
-            n, last_rank = n + 1, last_rank - 1
-        
-        msg = f'Moved {n} blocks to page {page}'
-        modeladmin.message_user(request, msg, messages.SUCCESS)
-        
-        return HttpResponseRedirect(request.get_full_path())
-    
-    template_name = 'admin/formative/move_page.html'
-    request.current_app = modeladmin.admin_site.name
-    min_page = max(block.min_allowed_page() for block in queryset)
-    last_page = queryset[0].form.blocks.aggregate(p=Max('page'))['p']
-    max_page = min(block.max_allowed_page(last_page) for block in queryset)
-    new = max_page == last_page
-    context = {
-        **modeladmin.admin_site.each_context(request),
-        'opts': modeladmin.model._meta,
-        'media': modeladmin.media,
-        'blocks': queryset,
-        'movable': queryset[0].form.status == Form.Status.DRAFT,
-        'form': MoveBlocksAdminForm(max_page, min_page=min_page, new_page=new),
-        'title': 'Move Blocks'
-    }
-    return TemplateResponse(request, template_name, context)
-
-
-EMAILS_PER_SECOND = 10
-
-@admin.action(description='Send an email to applicants')
-def send_email_action(modeladmin, request, queryset):
-    if '_send' in request.POST:
-        # TODO: celery task
-        subject = Template(request.POST['subject'])
-        content = Template(request.POST['content'])
-        
-        iterator = queryset.iterator()
-        batch, form, last_time, done, n = [], None, None, False, 0
-        while not done:
-            submission = next(iterator, None)
-            if not submission: done = True
-            else: batch.append(submission)
-            
-            if len(batch) == EMAILS_PER_SECOND or done:
-                if last_time:
-                    this_time = time.time()
-                    remaining = last_time + 1 - this_time
-                    if remaining > 0: time.sleep(remaining)
-                last_time = time.time()
-                
-                with mail.get_connection() as conn:
-                    for sub in batch:
-                        if not form: form = sub._get_form()
-                        context = {
-                            'submission': sub, 'form': form,
-                            'submission_link': submission_link(sub, form)
-                        }
-                        if sub._submitted: sub._update_context(form, context)
-                        
-                        n += 1
-                        send_email(content, sub._email, subject,
-                                   context=context, connection=conn)
-                batch = []
-        
-        msg = f'Emails sent to {n} recipients.'
-        modeladmin.message_user(request, msg, messages.SUCCESS)
-        
-        return HttpResponseRedirect(request.get_full_path())
-    
-    form = queryset[0]._get_form()
-    template_name = 'admin/formative/email_applicants.html'
-    request.current_app = modeladmin.admin_site.name
-    context = {
-        **modeladmin.admin_site.each_context(request),
-        'opts': modeladmin.model._meta,
-        'media': modeladmin.media,
-        'submissions': queryset,
-        'form': EmailAdminForm(form=form),
-        'email_templates': form.email_templates(),
-        'title': 'Email Applicants'
-    }
-    return TemplateResponse(request, template_name, context)
+from ..utils import send_email, submission_link, TabularExport
 
 
 class UserActionsMixin:
@@ -226,9 +134,145 @@ class FormActionsMixin:
             return TemplateResponse(request, template_name, context)
         
         return super().response_change(request, obj)
+    
+    @admin.action(description='Enable/disable a plugin')
+    def form_plugins(self, request, queryset):
+        if '_submit' in request.POST:
+            which, plugin, n = request.POST['which'], request.POST['plugin'], 0
+            for form in queryset:
+                if plugin not in form.get_available_plugins(): continue
+                if which == 'enable': form.add_plugins([plugin])
+                else: form.remove_plugins([plugin])
+                form.save()
+                n += 1
+            
+            msg = f"{capfirst(which)}d plugin '{plugin}' for {n} forms."
+            if n: self.message_user(request, msg, messages.SUCCESS)
+            
+            return HttpResponseRedirect(request.get_full_path())
+        
+        template_name = 'admin/formative/form_plugins.html'
+        request.current_app = self.admin_site.name
+        context = {
+            **self.admin_site.each_context(request),
+            'opts': self.model._meta, 'media': self.media,
+            'forms': queryset, 'title': 'Enable or Disable Plugin',
+            'form': FormPluginsAdminForm()
+        }
+        return TemplateResponse(request, template_name, context)
+
+
+class FormBlockActionsMixin:
+    @admin.action(description='Move to different page')
+    def move_blocks_action(self, request, queryset):
+        if '_move' in request.POST:
+            page, n = request.POST['page'], 0
+            blocks, cur_page = queryset[0].form.blocks, queryset[0].page
+            last_rank_q = blocks.filter(page=cur_page).aggregate(r=Max('_rank'))
+            last_rank = last_rank_q['r']
+            for block in cur_page and queryset.order_by('_rank') or []:
+                block = FormBlock.objects.get(pk=block.pk) # clear side effects;
+                block._rank = last_rank
+                block.save() # it's now at the end of its page; and
+                block.page, block._rank = page, None # then move to the new one,
+                block.save() # so that the others on the old page are left sorted
+                n, last_rank = n + 1, last_rank - 1
+            
+            msg = f'Moved {n} blocks to page {page}'
+            if n: self.message_user(request, msg, messages.SUCCESS)
+            
+            return HttpResponseRedirect(request.get_full_path())
+        
+        template_name = 'admin/formative/move_page.html'
+        request.current_app = self.admin_site.name
+        min_page = max(block.min_allowed_page() for block in queryset)
+        last_page = queryset[0].form.blocks.aggregate(p=Max('page'))['p']
+        max_page = min(block.max_allowed_page(last_page) for block in queryset)
+        new = max_page == last_page
+        context = {
+            **self.admin_site.each_context(request),
+            'opts': self.model._meta, 'media': self.media,
+            'blocks': queryset, 'title': 'Move Blocks',
+            'movable': queryset[0].form.status == Form.Status.DRAFT,
+            'form': MoveBlocksAdminForm(max_page,
+                                        min_page=min_page, new_page=new),
+        }
+        return TemplateResponse(request, template_name, context)
 
 
 class SubmissionActionsMixin:
+    EMAILS_PER_SECOND = 10
+    
+    @admin.action(description='Send an email to applicants')
+    def send_email(self, request, queryset):
+        if '_send' in request.POST:
+            # TODO: celery task
+            subject = Template(request.POST['subject'])
+            content = Template(request.POST['content'])
+            
+            iterator = queryset.iterator()
+            batch, form, last_time, done, n = [], None, None, False, 0
+            while not done:
+                submission = next(iterator, None)
+                if not submission: done = True
+                else: batch.append(submission)
+                
+                if len(batch) == self.EMAILS_PER_SECOND or done:
+                    if last_time:
+                        this_time = time.time()
+                        remaining = last_time + 1 - this_time
+                        if remaining > 0: time.sleep(remaining)
+                    last_time = time.time()
+                    
+                    with mail.get_connection() as conn:
+                        for sub in batch:
+                            if not form: form = sub._get_form()
+                            context = {
+                                'submission': sub, 'form': form,
+                                'submission_link': submission_link(sub, form)
+                            }
+                            if sub._submitted: sub._update_context(form,
+                                                                   context)
+                            n += 1
+                            send_email(content, sub._email, subject,
+                                       context=context, connection=conn)
+                    batch = []
+            
+            msg = f'Emails sent to {n} recipients.'
+            self.message_user(request, msg, messages.SUCCESS)
+            
+            return HttpResponseRedirect(request.get_full_path())
+        
+        form = queryset[0]._get_form()
+        template_name = 'admin/formative/email_applicants.html'
+        request.current_app = self.admin_site.name
+        context = {
+            **self.admin_site.each_context(request),
+            'opts': self.model._meta, 'media': self.media,
+            'submissions': queryset, 'title': 'Email Applicants',
+            'form': EmailAdminForm(form=form),
+            'email_templates': form.email_templates(),
+        }
+        return TemplateResponse(request, template_name, context)
+    
+    @admin.action(description='Export submissions as CSV')
+    def export_csv(self, request, queryset):
+        program_form = queryset.model._get_form()
+        if '_export' in request.POST:
+            filename = f'{program_form.slug}_export_selected.csv'
+            export = TabularExport(filename, program_form, queryset,
+                                   **request.POST)
+            return export.response(queryset)
+        
+        template_name = 'admin/formative/export_submissions.html'
+        context = {
+            **self.admin_site.each_context(request),
+            'opts': self.model._meta, 'media': self.media,
+            'submissions': queryset, 'title': 'Export Submissions',
+            'form': ExportAdminForm(program_form=program_form)
+        }
+        return TemplateResponse(request, template_name, context)
+    
     def change_view(self, request, object_id, **kwargs):
         action, kwargs = None, {}
         if '_submit_confirmed' in request.POST: action = 'submit'
