@@ -15,20 +15,21 @@ from polymorphic.admin import (PolymorphicParentModelAdmin,
                                PolymorphicChildModelAdmin,
                                PolymorphicChildModelFilter)
 import types
+from functools import partial
 from urllib.parse import unquote, parse_qsl
 
-from ..forms import ProgramAdminForm, FormAdminForm, StockBlockAdminForm, \
-    CustomBlockAdminForm, CollectionBlockAdminForm, SubmissionAdminForm, \
-    SubmissionItemAdminForm
+from ..forms import ProgramAdminForm, FormAdminForm, DependencyAdminForm, \
+    StockBlockAdminForm, CustomBlockAdminForm, CollectionBlockAdminForm, \
+    SubmissionAdminForm, SubmissionItemAdminForm
 from ..models import Program, Form, FormLabel, FormBlock, FormDependency, \
-    CustomBlock, CollectionBlock
+    CustomBlock, CollectionBlock, SubmissionRecord
 from ..filetype import FileType
 from ..plugins import get_matching_plugin
 from ..signals import register_program_settings, register_form_settings, \
     register_user_actions, form_published_changed
 from ..utils import submission_link
-from .actions import move_blocks_action, send_email_action, UserActionsMixin, \
-    FormActionsMixin, SubmissionActionsMixin
+from .actions import UserActionsMixin, FormActionsMixin,FormBlockActionsMixin, \
+    SubmissionActionsMixin
 
 
 class FormativeAdminSite(admin.AdminSite):
@@ -123,23 +124,26 @@ class FormAdmin(FormActionsMixin, admin.ModelAdmin):
     list_display = ('name', 'program', 'created', 'modified')
     list_filter = ('program',)
     form = FormAdminForm
+    actions = ['form_plugins']
     
     def get_changelist(self, request, **kwargs):
         return FormChangeList
     
     def get_fieldsets(self, request, obj=None):
-        fields = super().get_fields(request, obj)
+        opt_fields = super().get_fields(request, obj)
         main_fields = ['program', 'name', 'slug', 'status', 'hidden']
         
         main = (None, {'fields': main_fields[:3] + main_fields[4:]})
         if not obj: return [main]
         
-        for n in main_fields: fields.remove(n)
+        for n in main_fields: opt_fields.remove(n)
         main[1]['fields'] = main_fields
         
+        email_fields = ['email_names', 'emails']
         ret = [
             main,
-            ('Options', {'fields': fields}),
+            ('Options', {'fields': opt_fields}),
+            ('Emails', {'fields': email_fields})
         ]
         responses = register_form_settings.send(obj)
         for receiver, response in responses:
@@ -150,6 +154,7 @@ class FormAdmin(FormActionsMixin, admin.ModelAdmin):
     def get_readonly_fields(self, request, obj=None):
         fields = super().get_readonly_fields(request, obj)
         if obj:
+            fields += ('plugins',)
             if obj.status == Form.Status.DRAFT: fields += ('status',)
             else: fields += ('program', 'slug')
         return fields
@@ -164,6 +169,21 @@ class FormAdmin(FormActionsMixin, admin.ModelAdmin):
             if 'status' in form.changed_data: obj.completed = timezone.now()
         else: obj.completed = None
         
+        if 'email_names' in form.cleaned_data:
+            names = []
+            for n in form.cleaned_data['email_names'].split(','):
+                name = n.strip()
+                if not name: continue
+                names.append(name)
+                if name not in obj.emails():
+                    if 'emails' not in obj.options: obj.options['emails'] = {}
+                    obj.options['emails'][name] = {'subject': '', 'content': ''}
+            for name in obj.email_names():
+                if name in ('continue', 'confirmation'): continue
+                if name not in names: del obj.options['emails'][name]
+            if 'emails' in obj.options and not obj.options['emails']:
+                del obj.options['emails']
+        
         return obj
         
     def response_post_save_change(self, request, obj):
@@ -171,6 +191,13 @@ class FormAdmin(FormActionsMixin, admin.ModelAdmin):
         url = reverse('admin:%s_formblock_formlist' % (app_label,),
                       args=(obj.id,), current_app=self.admin_site.name)
         return HttpResponseRedirect(url)
+    
+    def has_delete_permission(self, request, obj=None):
+        if obj and obj.status != Form.Status.DRAFT: return False
+        return super().has_delete_permission(request, obj)
+    
+    def plugins(self, obj):
+        return ', '.join(obj.get_plugins()) or None
 
 
 @admin.register(FormLabel, site=site)
@@ -187,9 +214,21 @@ class FormLabelAdmin(admin.ModelAdmin):
         return formfield
 
 
+class FormDependencyFormSet(forms.BaseInlineFormSet):
+    def __init__(self, dependence=None, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.dependence = dependence
+    
+    def get_form_kwargs(self, index):
+        kwargs = super().get_form_kwargs(index)
+        kwargs['dependence'] = self.dependence
+        return kwargs
+
 class FormDependencyInline(admin.TabularInline):
     model = FormDependency
     extra = 0
+    formset = FormDependencyFormSet
+    form = DependencyAdminForm
     verbose_name_plural = 'dependency values'
     
     def has_add_permission(self, request, obj):
@@ -255,6 +294,11 @@ class FormBlockBase:
         
         return {}
     
+    def get_formset_kwargs(self, request, obj=None, *args):
+        kwargs = super().get_formset_kwargs(request, obj, *args)
+        if obj: kwargs['dependence'] = obj.dependence
+        return kwargs
+        
     def get_formsets_with_inlines(self, request, obj=None):
         for inline in self.get_inline_instances(request, obj):
             # only show the inline on the change form, not add:
@@ -350,7 +394,17 @@ class FormBlockBase:
         return super().has_add_permission(request)
     
     def has_delete_permission(self, request, obj=None):
-        return self.has_add_permission(request)
+        match = request.resolver_match
+        if 'form_id' not in match.kwargs:
+            return self.has_add_permission(request)
+        
+        try: form = Form.objects.get(id=match.kwargs['form_id'])
+        except Form.DoesNotExist: return False
+        if form.status != Form.Status.DRAFT: return False
+        page = request.GET.get('page')
+        if page is None or not page.isdigit(): page = None
+        if page and not int(page): return False
+        return True
 
 
 class PageListFilter(admin.SimpleListFilter):
@@ -360,16 +414,20 @@ class PageListFilter(admin.SimpleListFilter):
     
     def lookups(self, request, model_admin):
         qs = model_admin.get_queryset(request).distinct().order_by('page')
-        return [ (p, f'Page {p}' if p else 'Auto-created blocks')
-                 for p in qs.values_list('page', flat=True) ]
+        ret = list(qs.values_list('page', flat=True))
+        if len(ret) == 1: ret.append(1)
+        return [ (p, f'Page {p}' if p else 'Auto-created blocks') for p in ret ]
     
     def queryset(self, request, queryset):
+        val = self.value()
+        if val is None: return queryset
+        if not val.isdigit(): return queryset.none()
         return queryset.filter(page=self.value())
 
 
 @admin.register(FormBlock, site=site)
-class FormBlockAdmin(FormBlockBase, PolymorphicParentModelAdmin,
-                     DynamicArrayMixin):
+class FormBlockAdmin(FormBlockActionsMixin, FormBlockBase,
+                     PolymorphicParentModelAdmin, DynamicArrayMixin):
     child_models = (FormBlock, CustomBlock, CollectionBlock)
     list_display = ('_rank', 'name', 'block_type', 'dependence', 'labels_link')
     list_display_links = ('name',)
@@ -377,7 +435,7 @@ class FormBlockAdmin(FormBlockBase, PolymorphicParentModelAdmin,
     list_filter = (PageListFilter,)
     form = StockBlockAdminForm
     inlines = [FormDependencyInline]
-    actions = [move_blocks_action]
+    actions = ['move_blocks_action']
     sortable_by = ()
     polymorphic_list = True
     
@@ -460,10 +518,15 @@ class FormBlockAdmin(FormBlockBase, PolymorphicParentModelAdmin,
         return qs
     
     def get_changelist_form(self, request, **kwargs):
+        page = request.GET.get('page')
+        if page is None or not page.isdigit(): page = None
+        
         class HiddenWithHandleInput(forms.HiddenInput):
             template_name = 'admin/formative/widgets/hidden_with_handle.html'
         
-        kwargs['widgets'] = {'_rank': HiddenWithHandleInput}
+        if 'form_id' in request.resolver_match.kwargs and page and int(page):
+            kwargs['widgets'] = {'_rank': HiddenWithHandleInput}
+        else: kwargs['widgets'] = {'_rank': forms.HiddenInput}
         return super().get_changelist_form(request, **kwargs)
     
     def change_view(self, *args, **kwargs):
@@ -544,16 +607,17 @@ class CustomBlockAdmin(FormBlockChildAdmin, DynamicArrayMixin):
 @admin.register(CollectionBlock, site=site)
 class CollectionBlockAdmin(FormBlockChildAdmin, DynamicArrayMixin):
     form = CollectionBlockAdminForm
+    radio_fields = {'align_type': admin.VERTICAL}
     
     def get_fieldsets(self, request, obj=None):
         fieldsets = super().get_fieldsets(request, obj)
         fields = fieldsets[0][1]['fields']
         names = ['fixed', 'name1', 'name2', 'name3', 'has_file',
-                 'min_items', 'max_items', 'file_optional']
+                 'min_items', 'max_items', 'align_type', 'file_optional']
         
         if not obj: add = names[:5]
         elif obj.fixed: add = names[:4] + ['choices']
-        elif not obj.has_file: add = names[:6]
+        elif not obj.has_file: add = names[:8]
         else: add = names[:]
         
         main = ['name', 'page'] + add
@@ -601,12 +665,52 @@ class SubmittedListFilter(admin.SimpleListFilter):
         if self.value() == 'no': return queryset.filter(_submitted=None)
 
 
+class SubmissionRecordFormSet(forms.BaseModelFormSet):
+    @classmethod
+    def get_default_prefix(cls): return 'formative-submissionrecord'
+
+class SubmissionRecordInline(admin.TabularInline):
+    model = SubmissionRecord
+    formset = SubmissionRecordFormSet
+    exclude = ('program', 'form', 'submission')
+    readonly_fields = ('type', 'recorded', 'text', 'number', 'deleted')
+    
+    def has_add_permission(self, request, obj=None): return False
+    
+    def get_formset(self, request, obj=None, **kwargs):
+        return forms.modelformset_factory(self.model, **{
+            'form': self.form, 'formset': self.formset, 'fields': (),
+            'formfield_callback': partial(self.formfield_for_dbfield,
+                                          request=request),
+            'extra': 0, 'max_num': 0, 'can_delete': False, 'can_order': False
+        })
+
+
 class SubmissionAdmin(SubmissionActionsMixin, admin.ModelAdmin):
     list_display = ('_email', '_created', '_modified', '_submitted')
     list_filter = ('_email', SubmittedListFilter)
     readonly_fields = ('_submitted', 'items_index',)
     form = SubmissionAdminForm
-    actions = [send_email_action]
+    inlines = [SubmissionRecordInline]
+    actions = ['send_email', 'export_csv']
+    
+    def delete_queryset(self, request, queryset):
+        # something is up with model registry. manually delete the related items
+        related = queryset.model._get_form().item_model.objects
+        related.filter(_submission__in=queryset.values_list('pk',
+                                                            flat=True)).delete()
+        super().delete_queryset(request, queryset)
+    
+    def get_formsets_with_inlines(self, request, obj=None):
+        for inl in self.get_inline_instances(request, obj):
+            if not isinstance(inl, SubmissionRecordInline) or obj is not None:
+                yield inl.get_formset(request, obj), inl
+    
+    def get_formset_kwargs(self, request, obj, inline, prefix):
+        args = super().get_formset_kwargs(request, obj, inline, prefix)
+        for n in ('instance', 'save_as_new'): args.pop(n, None)
+        args['queryset'] = SubmissionRecord.objects.filter(submission=obj.pk)
+        return args
     
     @admin.display(description='items')
     def items_index(self, obj):
