@@ -1,5 +1,5 @@
 from django.utils.translation import gettext_lazy as _
-# from pikepdf import Pdf
+#from pikepdf import Pdf
 
 from . import FileType
 
@@ -14,8 +14,7 @@ class DocumentFile(FileType):
         try:
 #            with Pdf.open(path) as pdf:
 #                pages = len(pdf.pages)
-            pages = 1
-            ret.update(pages=pages)
+            ret.update(pages=1) #pages)
             return ret
         
         except:
