@@ -12,8 +12,10 @@ from ..filetype import FileType
 from ..stock import StockWidget
 from ..models import Program, Form, FormBlock, CustomBlock, CollectionBlock, \
     Submission, SubmissionItem
+from ..plugins import get_available_plugins
 from ..validators import validate_program_identifier, validate_form_identifier,\
     validate_formblock_identifier
+from ..utils import any_name_field
 
 
 class NullWidget(forms.Widget):
@@ -42,6 +44,8 @@ class JSONPseudoField(forms.Field):
 class NegatedBooleanField(forms.BooleanField):
     def prepare_value(self, value): return not value
     
+    def bound_data(self, data, initial): return not data
+    
     def to_python(self, value): return not super().to_python(value)
     
     def has_changed(self, initial, data):
@@ -59,7 +63,7 @@ class SplitDictWidget(forms.MultiWidget):
         
         subwidgets = {}
         for name, field in fields.items():
-            widget = forms.TextInput(attrs=attrs)
+            widget = field.widget
             if isinstance(field, forms.IntegerField):
                 if not attrs: int_attrs = {}
                 else: int_attrs = attrs.copy()
@@ -92,6 +96,8 @@ class SplitDictWidget(forms.MultiWidget):
             if name.startswith('min_') or name.startswith('max_'):
                 name = name[4:]
             if name.startswith('proc_'): name = name[5:]
+            if name.endswith('_subject') or name.endswith('_content'):
+                name = name[-7:]
             widget['short_name'] = name
         
         return context
@@ -375,13 +381,18 @@ class FormAdminForm(AdminJSONForm):
         required=False, widget=widgets.AdminTextareaWidget(attrs={'rows': 5}),
         label='thanks page text'
     )
+    email_names = forms.CharField(
+        required=False, label='custom email names',
+        help_text='If you want to define custom emails, enter a '
+                  'comma-separated list of names.'
+    )
     
     class Meta:
         static_fields = ('program', 'name', 'slug', 'status', 'hidden')
         json_fields = {'options': [
             'hidden', 'access_enable', 'review_pre', 'review_post',
             'submitted_review_pre', 'timed_completion', 'complete_submit_time',
-            'no_review_after_submit', 'thanks'
+            'no_review_after_submit', 'thanks', 'emails'
         ]}
         dynamic_fields = True
     
@@ -404,8 +415,50 @@ class FormAdminForm(AdminJSONForm):
             del self.fields['status']
             for n in ('access_enable', 'review_pre', 'review_post', 'thanks',
                       'submitted_review_pre', 'timed_completion',
-                      'complete_submit_time', 'no_review_after_submit'):
+                      'complete_submit_time', 'no_review_after_submit',
+                      'email_names'):
                 del self.fields[n]
+        else:
+            custom_emails = {}
+            email_names = []
+            for n in program_form.email_names():
+                fields = {'subject': forms.CharField(),
+                          'content': forms.CharField(widget=forms.Textarea)}
+                custom_emails[n] = SplitDictField(fields,
+                    required=False, widget=SplitDictWidget(fields)
+                )
+                if n not in ('continue', 'confirmation'):
+                    email_names.append(n)
+            self.fields['emails'] = SplitDictField(custom_emails,
+                required=False, widget=SplitDictWidget(custom_emails),
+                help_text='Non-custom emails: leave blank to use the ' \
+                          'default text.'
+            )
+            self.fields['email_names'].initial = ','.join(email_names)
+
+
+class DependencyAdminForm(forms.ModelForm):
+    value = forms.ChoiceField(choices=[('yes', 'yes'), ('no', 'no')])
+    
+    def __init__(self, dependence=None, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        
+        if not dependence:
+            self.fields['value'] = forms.CharField(max_length=64,
+                                                   required=False)
+            return
+        
+        bool_str = {True: 'yes', False: 'no'}
+        if dependence.block_type() == 'stock':
+            vals = [ bool_str[v] if type(v) != str else v
+                     for v in dependence.stock.conditional_values() ]
+            self.fields['value'].choices = [ (n, n) for n in vals ]
+        elif dependence.block_type() == 'custom':
+            if dependence.type == CustomBlock.InputType.CHOICE:
+                vals = dependence.choices(include_empty=True)
+                self.fields['value'].choices = [('', '[none]')]
+                self.fields['value'].choices += [ (n, n) for n in vals ]
+        self.fields['value'].required = False
 
 
 class FormBlockAdminForm(forms.ModelForm):
@@ -590,12 +643,13 @@ class CollectionBlockAdminForm(FormBlockAdminForm, AdminJSONForm):
     )
     
     class Meta:
-        exclude = ('form', '_rank', 'align_type')
+        exclude = ('form', '_rank')
         static_fields = ('name', 'page', 'fixed', 'name1', 'name2', 'name3',
                          'has_file', 'min_items', 'max_items', 'file_optional',
-                         'dependence', 'negate_dependencies')
+                         'dependence', 'negate_dependencies', 'align_type')
         json_fields = {'options': ['no_review']}
         dynamic_fields = True
+        labels = {'align_type': 'display type'}
     
     def init_file_fields(self, block):
         admin_fields, total_fields = {}, {}
@@ -685,6 +739,13 @@ class CollectionBlockAdminForm(FormBlockAdminForm, AdminJSONForm):
                 self.add_error('name1', msg)
             return cleaned_data
         
+        form = self.instance.form
+        if 'align_type' in cleaned_data and form.status == Form.Status.DRAFT:
+            if cleaned_data['align_type'] == CollectionBlock.AlignType.STACKED:
+                if not cleaned_data['name1']:
+                    msg = 'Stacked display must have at least one text input'
+                    self.add_error('name1', msg)
+        
         if 'autoinit_filename' in cleaned_data:
             field = cleaned_data['autoinit_filename']
             if field not in self.instance.collection_fields():
@@ -727,6 +788,33 @@ class EmailAdminForm(forms.Form):
         self.fields['name'].choices.append(('', '[untitled]'))
 
 
+class ExportAdminForm(forms.Form):
+    def __init__(self, program_form=None, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        
+        if not program_form: return
+        for block in program_form.submission_blocks():
+            name = 'block_' + block.name
+            self.fields[name] = forms.BooleanField(required=False, initial=True,
+                                                   label=block.name)
+        
+        choices = [('no', 'not included'), ('repeat', 'repeated columns'),
+                   ('combine', 'in one column')]
+        for block in program_form.collections():
+            name = 'collection_' + block.name
+            self.fields[name] = forms.ChoiceField(label=block.name,
+                choices=choices, initial='combine' if block.fixed else 'no'
+            )
+            for field in block.collection_fields():
+                name = f'cfield_{block.name}.{field}'
+                self.fields[name] = forms.BooleanField(label=field,
+                    required=False, initial=block.fixed)
+            if block.has_file:
+                self.fields[f'cfield_{block.name}._file'] = forms.BooleanField(
+                    label='file URL', required=False
+                )
+
+
 class MoveBlocksAdminForm(forms.Form):
     page = forms.ChoiceField(
         choices=(), widget=forms.Select(attrs={'style': 'padding: 4px'})
@@ -740,5 +828,14 @@ class MoveBlocksAdminForm(forms.Form):
         self.fields['page'].choices = choices
 
 
+class FormPluginsAdminForm(forms.Form):
+    plugin = forms.ChoiceField(required=True,
+                               choices=[('', '-')]+
+                                       [ (n, p.name) for n, p
+                                         in get_available_plugins().items() ])
+    which = forms.ChoiceField(choices=[('enable', 'Enable plugin'),
+                                       ('disable', 'Disable plugin')])
+
+
 class UserImportForm(forms.Form):
     csv_file = forms.FileField()
