@@ -147,8 +147,7 @@ class Form(AutoSlugModel):
                                               related_query_name='_item'))
         ]
         
-        field_blocks = { b.name: b for b in
-                         self.custom_blocks().filter(page=0, _rank__gt=1) }
+        field_blocks = { b.name: b for b in self.collection_field_blocks() }
         for n in names:
             # look for a CustomBlock with the same name on page 0 (hidden)
             if n in field_blocks: block = field_blocks[n]
@@ -175,6 +174,8 @@ class Form(AutoSlugModel):
         else: cache.incr('models_version')
     
     def publish_model(self, model, admin=None):
+        from ..signals import all_forms_publish
+        
         with connection.schema_editor() as editor:
             editor.create_model(model)
         ctype = ContentType(app_label=model._meta.app_label,
@@ -182,12 +183,16 @@ class Form(AutoSlugModel):
         ctype.save()
         ContentType.objects.clear_cache()
         
+        all_forms_publish.send(self, content_type=ctype)
         self.cache_dirty()
     
     def unpublish_model(self, model):
-        self.cache_dirty()
+        from ..signals import all_forms_unpublish
         
+        self.cache_dirty()
         ctype = ContentType.objects.get_for_model(model)
+        all_forms_unpublish.send(self, content_type=ctype)
+        
         ctype.delete()
         ContentType.objects.clear_cache()
         with connection.schema_editor() as editor:
@@ -227,10 +232,8 @@ class Form(AutoSlugModel):
         self.save()
     
     def get_available_plugins(self):
-        from ..plugins import get_all_plugins
-        
-        return { plugin.module: plugin for plugin in get_all_plugins(self)
-                 if not plugin.name.startswith('.') }
+        from ..plugins import get_available_plugins
+        return get_available_plugins(self)
     
     def get_plugins(self):
         if 'plugins' in self.options: return self.options['plugins']
@@ -239,7 +242,8 @@ class Form(AutoSlugModel):
     def add_plugins(self, plugins):
         available = self.get_available_plugins()
         
-        enable = [ p for p in plugins if p in available ]
+        enabled = self.get_plugins()
+        enable = [ p for p in plugins if p in available and p not in enabled ]
         for plugin in enable:
             if hasattr(available[plugin].app, 'installed'):
                 getattr(available[plugin].app, 'installed')(self)
@@ -249,12 +253,13 @@ class Form(AutoSlugModel):
     
     def remove_plugins(self, plugins):
         available = self.get_available_plugins()
-
-        for plugin in plugins:
+        
+        enabled = self.get_plugins()
+        for plugin in [ p for p in plugins if p in enabled ]:
             if hasattr(available[plugin].app, 'uninstalled'):
                 getattr(available[plugin].app, 'uninstalled')(self)
 
-        new_plugins = [ p for p in self.options['plugins'] if p not in plugins ]
+        new_plugins = [ p for p in enabled if p not in plugins ]
         self.options['plugins'] = new_plugins
     
     def default_text_label_style(self):
@@ -272,6 +277,9 @@ class Form(AutoSlugModel):
         blocks = self.blocks.not_instance_of(CollectionBlock)
         return blocks.exclude(page=0, _rank__gt=0)
     
+    def collection_field_blocks(self):
+        return self.custom_blocks().filter(page=0, _rank__gt=1)
+    
     def collections(self, name=None):
         blocks = CollectionBlock.objects.filter(form=self)
         if name: return blocks.filter(name=name).non_polymorphic()
@@ -547,12 +555,14 @@ class FormBlock(PolymorphicModel, RankedModel):
         return query.values_list('id', flat=True)
     
     def min_allowed_page(self):
+        if not self.page: return 0
         min_page = 1
         
         if self.dependence: min_page = self.dependence.page + 1
         return min_page
     
     def max_allowed_page(self, last_page=None):
+        if not self.page: return 0
         if last_page is None:
             last_page = self.form.blocks.aggregate(p=Max('page'))['p'] or 1
         max_page = last_page
@@ -596,7 +606,10 @@ class CustomBlock(FormBlock):
             kwargs['max_chars'] = cls.DEFAULT_TEXT_MAXLEN
         return cls(*args, **kwargs, type=cls.InputType.TEXT)
     
-    def choices(self):
+    def choices(self, include_empty=False):
+        if include_empty:
+            if 'choices' in self.options: return self.options['choices']
+            return []
         if 'choices' not in self.options or not self.options['choices']:
             raise FieldError('choices not defined')
         
@@ -705,8 +718,8 @@ class CollectionBlock(FormBlock):
     FIXED_CHOICE_VAL_MAXLEN = 100
     
     class AlignType(models.TextChoices):
-        HORIZONTAL = 'horizontal', _('horizontal')
-        VERTICAL = 'vertical', _('vertical')
+        TABULAR = 'tabular', _('tabular')
+        STACKED = 'stacked', _('stacked')
     
     block = models.OneToOneField(FormBlock, on_delete=models.CASCADE,
                                  parent_link=True, primary_key=True)
@@ -720,7 +733,7 @@ class CollectionBlock(FormBlock):
     name2 = models.CharField(max_length=32, default='', blank=True)
     name3 = models.CharField(max_length=32, default='', blank=True)
     align_type = models.CharField(max_length=16, choices=AlignType.choices,
-                                  default=AlignType.HORIZONTAL)
+                                  default=AlignType.TABULAR)
     
     def fields(self):
         return []
@@ -733,6 +746,9 @@ class CollectionBlock(FormBlock):
         
         return fields
     
+    def tabular(self):
+        return self.align_type == self.AlignType.TABULAR
+    
     def max_filesize(self):
         if 'max_filesize' in self.options: return self.options['max_filesize']
         return None # TODO: overall default max
